/**
 *
 * Copyright 2004 Protique Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 **/
package org.codehaus.activemq.jca;

import junit.framework.TestCase;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.codehaus.activemq.broker.BrokerContainer;
import org.codehaus.activemq.broker.impl.BrokerContainerImpl;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.jms.core.JmsTemplate;

import javax.jms.ConnectionFactory;

/**
 * @author srt
 * @version $Id: SpringTemplateAndJCATest.java,v 1.1 2004/12/20 10:23:35 jstrachan Exp $
 */
public class SpringTemplateAndJCATest extends TestCase {
    private static Log logger = LogFactory.getLog(SpringTemplateAndJCATest.class);

    private ConfigurableApplicationContext applicationContext;
    private ConnectionFactory connectionFactory;
    private BrokerContainer broker;

    public void testRun() {
        for (int i = 0; i < 10; i++) {
            String text;

            text = "Message " + i;
            logger.info("Sending " + text);
            sendMessage(text);

        }
        try {
            Thread.sleep(2000);
        }
        catch (InterruptedException e) {
        }
    }


    public void sendMessage(String text) {
        JmsTemplate template = new JmsTemplate(connectionFactory);
        template.setPubSubDomain(true);
        template.convertAndSend("myTopic", text);
    }

    protected void setUp() throws Exception {
        // lets create a JMS broker
        broker = new BrokerContainerImpl();
        broker.addConnector("tcp://localhost:61616");
        broker.start();

        applicationContext = new ClassPathXmlApplicationContext("org/codehaus/activemq/jca/spring-topic.xml");
        connectionFactory = (ConnectionFactory) applicationContext.getBean("jmsFactory");

        assertTrue("Should have found a non-null connection factory", connectionFactory != null);
    }

    protected void tearDown() throws Exception {
        if (applicationContext != null) {
            System.out.println("Closing the application context");
            applicationContext.close();
        }
        broker.stop();
    }
}
