/*
 * 
 * Copyright 2004 Hiram Chirino
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 */
package org.codehaus.activemq.store;

import javax.jms.JMSException;

import org.codehaus.activemq.message.ActiveMQMessage;
import org.codehaus.activemq.message.MessageAck;
import org.codehaus.activemq.service.MessageIdentity;
import org.codehaus.activemq.service.QueueMessageContainer;

/**
 * A simple proxy that delegates to another MessageStore.
 */
public class ProxyMessageStore implements MessageStore {

    final MessageStore delegate;
    
    public ProxyMessageStore(MessageStore delegate) {
        this.delegate = delegate;
    }
    
    public MessageStore getDelegate() {
        return delegate;
    }

    public void addMessage(ActiveMQMessage message) throws JMSException {
        delegate.addMessage(message);
    }
    public ActiveMQMessage getMessage(MessageIdentity identity) throws JMSException {
        return delegate.getMessage(identity);
    }
    public void recover(QueueMessageContainer container) throws JMSException {
        delegate.recover(container);
    }
    public void removeAllMessages() throws JMSException {
        delegate.removeAllMessages();
    }
    public void removeMessage(MessageAck ack) throws JMSException {
        delegate.removeMessage(ack);
    }
    public void start() throws JMSException {
        delegate.start();
    }
    public void stop() throws JMSException {
        delegate.stop();
    }
}
