/** 
 * 
 * Copyright 2004 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.codehaus.activemq.store;

import javax.jms.JMSException;
import javax.transaction.xa.XAException;

import org.codehaus.activemq.message.ActiveMQMessage;
import org.codehaus.activemq.message.ActiveMQXid;
import org.codehaus.activemq.message.MessageAck;
import org.codehaus.activemq.service.Service;

/**
 * Represents the durable store of the commit/rollback operations taken against the
 * broker.
 *
 * @version $Revision: 1.1 $
 */
public interface TransactionStore extends Service {

    public void prepare(Object txid) throws XAException;    
    public void commit(Object txid, boolean wasPrepared) throws XAException;    
    public void rollback(Object txid) throws XAException; 

    public static interface RecoveryListener {
        public void recover(ActiveMQXid xid, ActiveMQMessage addedMessages[], MessageAck aks[]) throws JMSException,XAException;        
    } 
    
    /**
     * 
     * 
     * @param listener
     * @throws JMSException
     */
    public void recover(RecoveryListener listener) throws XAException;
    
}
