/** 
 * 
 * Copyright 2004 Hiram Chirino
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.codehaus.activemq.transport.activeio;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;

import javax.jms.JMSException;

import org.activeio.AsynchChannelServer;
import org.activeio.ChannelFactory;
import org.codehaus.activemq.io.WireFormat;
import org.codehaus.activemq.transport.TransportServerChannel;
import org.codehaus.activemq.transport.TransportServerChannelFactory;
import org.codehaus.activemq.util.JMSExceptionHelper;

/**
 * A tcp implementation of a TransportServerChannelFactory
 * 
 * @version $Revision: 1.1 $
 */
public class ActiveIOTransportServerChannelFactory implements TransportServerChannelFactory {

    /**
     * Bind a ServerChannel to an address
     *O
     * @param wireFormat
     * @param bindAddress
     * @return the TransportChannel bound to the remote node
     * @throws JMSException
     */
    public TransportServerChannel create(WireFormat wireFormat, URI bindAddress) throws JMSException {
        AsynchChannelServer server = createAsynchChannelServer(bindAddress);
        return new ActiveIOTransportServerChannel(wireFormat, server);
    }

    /**
     * @param bindAddress
     * @return
     */
    private AsynchChannelServer createAsynchChannelServer(URI bindAddress) throws JMSException {
        try {
            bindAddress = URIConverter.convert(bindAddress);
            AsynchChannelServer server = new ChannelFactory().bindAsynchChannel(bindAddress);
            return server;
        } catch (IOException e) {
            throw JMSExceptionHelper.newJMSException(e.getMessage(), e);
        } catch (URISyntaxException e) {
            throw JMSExceptionHelper.newJMSException(e.getMessage(), e);
        }
    }

}