/**
 * 
 * Copyright 2004 Hiram Chirino
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.codehaus.activemq.store.jdbm;

import java.util.Map;

import javax.jms.JMSException;

import org.apache.geronimo.gbean.GBeanInfo;
import org.apache.geronimo.gbean.GBeanInfoBuilder;
import org.apache.geronimo.gbean.GBeanLifecycle;
import org.apache.geronimo.system.serverinfo.ServerInfo;
import org.codehaus.activemq.service.QueueMessageContainer;
import org.codehaus.activemq.service.TopicMessageContainer;
import org.codehaus.activemq.store.MessageStore;
import org.codehaus.activemq.store.PersistenceAdapter;
import org.codehaus.activemq.store.TopicMessageStore;
import org.codehaus.activemq.store.TransactionStore;

/**
 *
 */
public class JdbmPersistenceAdapterGBean implements GBeanLifecycle, PersistenceAdapter {

    private JdbmPersistenceAdapter pa;
    private String directory;
    private final ServerInfo serverInfo;
    
    public JdbmPersistenceAdapterGBean() {
        serverInfo=null;
    }
    
    public JdbmPersistenceAdapterGBean(ServerInfo serverInfo, String directory) {
        assert serverInfo!=null;
        assert directory!=null;
        
        this.serverInfo = serverInfo;
        this.directory = directory;
    }
    
    public void setDirectory(String directory) {
        this.directory = directory;
    }

    public String getDirectory() {
        return directory;
    }
    
    public void doStart() throws Exception {
        pa = new JdbmPersistenceAdapter(serverInfo.resolve(directory));
        pa.start();
    }

    public void doStop() throws Exception {
        pa.stop();
        pa = null;
    }
    
    public void doFail() {
    }
    
    public static final GBeanInfo GBEAN_INFO;
    static {
        GBeanInfoBuilder infoFactory = new GBeanInfoBuilder("ActiveMQ JDBM Persistence", JdbmPersistenceAdapterGBean.class, "JMSPersistence");
        infoFactory.addReference("ServerInfo", ServerInfo.class);
        infoFactory.addAttribute("dataDirectory", String.class, true);
        
        infoFactory.addInterface(PersistenceAdapter.class);
        infoFactory.setConstructor(new String[]{"ServerInfo", "dataDirectory"});
        GBEAN_INFO = infoFactory.getBeanInfo();
    }
    public static GBeanInfo getGBeanInfo() {
        return GBEAN_INFO;
    }

    public void beginTransaction() throws JMSException {
        pa.beginTransaction();
    }
    public void commitTransaction() throws JMSException {
        pa.commitTransaction();
    }
    public QueueMessageContainer createQueueMessageContainer(String destinationName) throws JMSException {
        return pa.createQueueMessageContainer(destinationName);
    }
    public MessageStore createQueueMessageStore(String destinationName) throws JMSException {
        return pa.createQueueMessageStore(destinationName);
    }
    public TopicMessageContainer createTopicMessageContainer(String destinationName) throws JMSException {
        return pa.createTopicMessageContainer(destinationName);
    }
    public TopicMessageStore createTopicMessageStore(String destinationName) throws JMSException {
        return pa.createTopicMessageStore(destinationName);
    }
    public TransactionStore createTransactionStore() throws JMSException {
        return pa.createTransactionStore();
    }
    public Map getInitialDestinations() {
        return pa.getInitialDestinations();
    }
    public void rollbackTransaction() {
        pa.rollbackTransaction();
    }
    public void start() throws JMSException {
    }
    public void stop() throws JMSException {
    }
}
